@@ -1,4 +1,3 @@
-# $Id: facter.rb 179 2006-09-29 16:13:12Z luke $
 #--
 # Copyright 2006 Luke Kanies <luke@madstop.com>
 # 
@@ -22,7 +21,7 @@ class Facter
     include Comparable
     include Enumerable
 
-    FACTERVERSION = '1.3.5'
+    FACTERVERSION = '1.3.7'
 	# = Facter
     # Functions as a hash of 'facts' you might care about about your
     # system, such as mac address, IP address, Video card, etc.
@@ -37,7 +36,14 @@ class Facter
     #
 
 
-    @@facts = {}
+    @@facts = Hash.new { |hash, key|
+        key = key.to_s.downcase.intern
+        if hash.include?(key)
+            hash[key]
+        else
+            nil
+        end
+    }
     GREEN = "[0;32m"
     RESET = "[0m"
     @@debug = 0
@@ -47,12 +53,12 @@ class Facter
 	# module methods
 
     # Return the version of the library.
-    def Facter.version
+    def self.version
         return FACTERVERSION
     end
 
     # Add some debugging
-    def Facter.debug(string)
+    def self.debug(string)
         if string.nil?
             return
         end
@@ -63,22 +69,17 @@ class Facter
 
     # Return a fact object by name.  If you use this, you still have to call
     # 'value' on it to retrieve the actual value.
-    def Facter.[](name)
-        name = name.to_s.downcase
+    def self.[](name)
         @@facts[name]
     end
 
     # Add a resolution mechanism for a named fact.  This does not distinguish
     # between adding a new fact and adding a new way to resolve a fact.
-    def Facter.add(name, &block)
+    def self.add(name, &block)
         fact = nil
-        dcname = name.to_s.downcase
 
-        if @@facts.include?(dcname)
-            fact = @@facts[dcname]
-        else
-            Facter.new(dcname)
-            fact = @@facts[dcname]
+        unless fact = @@facts[name]
+            fact = Facter.new(name)
         end
 
         unless block
@@ -98,7 +99,7 @@ class Facter
                 if fact.suitable?
                     value = fact.value
                     unless value.nil?
-                        yield name, fact.value
+                        yield name.to_s, fact.value
                     end
                 end
             }
@@ -113,7 +114,7 @@ class Facter
                 name = name.to_s.sub(/\?$/,'')
             end
 
-            if fact = self[name]
+            if fact = @@facts[name]
                 if question
                     value = fact.value.downcase
                     args.each do |arg|
@@ -135,14 +136,13 @@ class Facter
     end
 
     # Clear all facts.  Mostly used for testing.
-    def Facter.clear
-        Facter.reset
+    def self.clear
         Facter.flush
-        @@facts.clear
+        Facter.reset
     end
 
 	# Set debugging on or off.
-	def Facter.debugging(bit)
+	def self.debugging(bit)
 		if bit
             case bit
             when TrueClass: @@debug = 1
@@ -168,38 +168,37 @@ class Facter
 	end
 
     # Flush all cached values.
-    def Facter.flush
-        @@facts.each { |fact| fact.flush }
+    def self.flush
+        @@facts.each { |name, fact| fact.flush }
     end
 
     # Return a list of all of the facts.
-    def Facter.list
+    def self.list
         return @@facts.keys
     end
 
     # Remove them all.
-    def Facter.reset
-        @@facts.each { |name,fact|
-            @@facts.delete(name)
-        }
+    def self.reset
+        @@facts.clear
     end
 
     # Return a hash of all of our facts.
-    def Facter.to_hash(*tags)
+    def self.to_hash(*tags)
         @@facts.inject({}) do |h, ary|
             if ary[1].suitable? and (tags.empty? or ary[1].tagged?(*tags))
                 value = ary[1].value
                 if value
-                    h[ary[0]] = value
+                    # For backwards compatibility, convert the fact name to a string.
+                    h[ary[0].to_s] = value
                 end
             end
             h
         end
     end
 
-    def Facter.value(name)
-        if @@facts.include?(name.to_s.downcase)
-            @@facts[name.to_s.downcase].value
+    def self.value(name)
+        if fact = @@facts[name]
+            fact.value
         else
             nil
         end
@@ -217,9 +216,9 @@ class Facter
 
     # Create a new fact, with no resolution mechanisms.
     def initialize(name)
-        @name = name.downcase if name.is_a? String
+        @name = name.to_s.downcase.intern
         if @@facts.include?(@name)
-            raise ArgumentError, "A fact named %s already exists" % name
+            raise ArgumentError, "A fact named %s already exists" % @name
         else
             @@facts[@name] = self
         end
@@ -385,26 +384,37 @@ class Facter
     class Resolution
         attr_accessor :interpreter, :code, :name, :fact
 
+        def Resolution.have_which
+            if @have_which.nil?
+                %x{which which 2>/dev/null}
+                @have_which = ($? == 0)
+            end
+            @have_which
+        end
+
         # Execute a chunk of code.
         def Resolution.exec(code, interpreter = "/bin/sh")
             if interpreter == "/bin/sh"
                 binary = code.split(/\s+/).shift
 
-                path = nil
-                if binary !~ /^\//
-                    path = %x{which #{binary} 2>/dev/null}.chomp
-                    if path == ""
-                        # we don't have the binary necessary
+                if have_which
+                    path = nil
+                    if binary !~ /^\//
+                        path = %x{which #{binary} 2>/dev/null}.chomp
+                        if path == ""
+                            # we don't have the binary necessary
+                            return nil
+                        end
+                    else
+                        path = binary
+                    end
+
+                    unless FileTest.exists?(path)
+                        # our binary does not exist
                         return nil
                     end
-                else
-                    path = binary
                 end
 
-                unless FileTest.exists?(path)
-                    # our binary does not exist
-                    return nil
-                end
                 out = nil
                 begin
                     out = %x{#{code}}.chomp
@@ -571,16 +581,16 @@ class Facter
     end
 
     # Load all of the default facts
-    def Facter.loadfacts
-        Facter.add("FacterVersion") do
+    def self.loadfacts
+        Facter.add(:facterversion) do
             setcode { FACTERVERSION.to_s }
         end
 
-        Facter.add("RubyVersion") do
+        Facter.add(:rubyversion) do
             setcode { RUBY_VERSION.to_s }
         end
 
-        Facter.add("PuppetVersion") do
+        Facter.add(:puppetversion) do
             setcode {
                 begin
                     require 'puppet'
@@ -600,11 +610,11 @@ class Facter
             end
         end
 
-        Facter.add("Kernel") do
+        Facter.add(:kernel) do
             setcode 'uname -s'
         end
 
-        Facter.add("KernelRelease") do
+        Facter.add(:kernelrelease) do
             setcode 'uname -r'
         end
 
@@ -626,23 +636,21 @@ class Facter
             end
         end
 
-        Facter.add("OperatingSystem") do
+        Facter.add(:operatingsystem) do
             # Default to just returning the kernel as the operating system
-            setcode do Facter["Kernel"].value end
+            setcode do Facter[:kernel].value end
         end
 
-        Facter.add("OperatingSystemRelease") do
-            setcode do Facter["KernelRelease"].value end
+        Facter.add(:operatingsystemrelease) do
+            setcode do Facter[:kernelrelease].value end
         end
 
-        Facter.add("OperatingSystem") do
-            #obj.os = "Linux"
+        Facter.add(:operatingsystem) do
             confine :kernel => :sunos
             setcode do "Solaris" end
         end
 
-        Facter.add("OperatingSystem") do
-            #obj.os = "Linux"
+        Facter.add(:operatingsystem) do
             confine :kernel => :linux
             setcode do
                 if FileTest.exists?("/etc/debian_version")
@@ -664,11 +672,11 @@ class Facter
             end
         end
 
-        Facter.add("HardwareModel") do
+        Facter.add(:hardwaremodel) do
             setcode 'uname -m'
         end
 
-        Facter.add("Architecture") do
+        Facter.add(:architecture) do
             confine :kernel => :linux
             setcode do
                 model = Facter.hardwaremodel
@@ -683,13 +691,15 @@ class Facter
             end
         end
 
-        Facter.add("CfKey") do
+        Facter.add(:Cfkey) do
             setcode do
                 value = nil
                 ["/usr/local/etc/cfkey.pub",
                     "/etc/cfkey.pub",
                     "/var/cfng/keys/localhost.pub",
-                    "/var/cfengine/ppkeys/localhost.pub"
+                    "/var/cfengine/ppkeys/localhost.pub",
+                    "/var/lib/cfengine/ppkeys/localhost.pub",
+                    "/var/lib/cfengine2/ppkeys/localhost.pub"
                 ].each { |file|
                     if FileTest.file?(file)
                         File.open(file) { |openfile|
@@ -709,7 +719,7 @@ class Facter
             end
         end
 
-        Facter.add("Domain") do
+        Facter.add(:domain) do
             setcode do
                 # First force the hostname to be checked
                 Facter.hostname
@@ -723,7 +733,7 @@ class Facter
             end
         end
         # Look for the DNS domain name command first.
-        Facter.add("Domain") do
+        Facter.add(:domain) do
             setcode do
                 domain = Resolution.exec('dnsdomainname') or nil
                 # make sure it's a real domain
@@ -734,7 +744,7 @@ class Facter
                 end
             end
         end
-        Facter.add("Domain") do
+        Facter.add(:domain) do
             setcode do
                 domain = Resolution.exec('domainname') or nil
                 # make sure it's a real domain
@@ -745,7 +755,7 @@ class Facter
                 end
             end
         end
-        Facter.add("Domain") do
+        Facter.add(:domain) do
             setcode do
                 value = nil
                 if FileTest.exists?("/etc/resolv.conf")
@@ -773,7 +783,7 @@ class Facter
                 end
             end
         end
-        Facter.add("Hostname") do
+        Facter.add(:hostname) do
             setldapname "cn"
             setcode do
                 hostname = nil
@@ -805,7 +815,7 @@ class Facter
             end
         end
 
-        Facter.add("IPAddress") do
+        Facter.add(:ipaddress) do
             setldapname "iphostnumber"
             setcode do
                 require 'resolv'
@@ -826,13 +836,13 @@ class Facter
                 end
             end
         end
-        Facter.add("IPAddress") do
+        Facter.add(:ipaddress) do
             setcode do
                 if hostname = Facter.hostname
                     # we need Hostname to exist for this to work
                     host = nil
-                    if host = Resolution.exec("host #{hostname}") and host.chomp.split(/\s/)
-
+                    if host = Resolution.exec("host #{hostname}")
+                        host = host.chomp.split(/\s/)
                         if defined? list[-1] and
                                 list[-1] =~ /[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+/
                             list[-1]
@@ -855,7 +865,7 @@ class Facter
                         filepath = File.join(dir,file)
                         if FileTest.file?(filepath)
                             begin
-                                value = File.open(filepath).read.chomp
+                                value = File.open(filepath).read.chomp.split(/\s+/)[1]
                             rescue
                                 value = nil
                             end
@@ -866,19 +876,17 @@ class Facter
             } # end of hash each
         } # end of dir each
 
-        Facter.add("UniqueId") do
+        Facter.add(:uniqueid) do
             setcode 'hostid',  '/bin/sh'
-            #confine :kernel => %w{Solaris Linux}
             confine :operatingsystem => %w{Solaris Linux Fedora RedHat CentOS SuSE Debian Gentoo}
         end
 
-        Facter.add("HardwareISA") do
+        Facter.add(:hardwareisa) do
             setcode 'uname -p', '/bin/sh'
             confine :operatingsystem => %w{Solaris Linux Fedora RedHat CentOS SuSE Debian Gentoo}
         end
 
-        Facter.add("MacAddress") do
-            #confine :kernel => %w{Solaris Linux}
+        Facter.add(:macaddress) do
             confine :operatingsystem => %w{Solaris Linux Fedora RedHat CentOS SuSE Debian Gentoo}
             setcode do
                 ether = []
@@ -886,11 +894,11 @@ class Facter
                 output.each {|s|
                              ether.push($1) if s =~ /(?:ether|HWaddr) (\w{1,2}:\w{1,2}:\w{1,2}:\w{1,2}:\w{1,2}:\w{1,2})/
                             }
-                ether.join(" ")
+                ether[0]
             end
         end
 
-        Facter.add("MacAddress") do
+        Facter.add(:macaddress) do
             confine :kernel => :darwin
             setcode do
                 ether = nil
@@ -905,8 +913,9 @@ class Facter
 
                 ether
             end
+        end
 
-         Facter.add("IPAddress") do
+         Facter.add(:ipaddress) do
             confine :kernel => :linux
             setcode do
                 ip = nil
@@ -925,9 +934,8 @@ class Facter
                 ip
             end
         end
-        end
-        Facter.add("IPAddress") do
-            confine :kernel => %w{FreeBSD NetBSD OpenBSD solaris darwin}
+        Facter.add(:ipaddress) do
+            confine :kernel => %w{FreeBSD NetBSD OpenBSD solaris}
             setcode do
                 ip = nil
                 output = %x{/sbin/ifconfig}
@@ -945,19 +953,49 @@ class Facter
                 ip
             end
         end
-        Facter.add("Hostname") do
+        Facter.add(:ipaddress) do
+            confine :kernel => %w{darwin}
+            setcode do
+                ip = nil
+                iface = ""
+                output = %x{/usr/sbin/netstat -rn}
+                if output =~ /^default\s*\S*\s*\S*\s*\S*\s*\S*\s*(\S*).*/
+                  iface = $1
+                else
+                  warn "Could not find a default route. Using first non-loopback interface"
+                end
+                if(iface != "")
+                  output = %x{/sbin/ifconfig #{iface}}                  
+                else
+                  output = %x{/sbin/ifconfig}
+                end
+
+                output.split(/^\S/).each { |str|
+                    if str =~ /inet ([0-9]+\.[0-9]+\.[0-9]+\.[0-9]+)/
+                        tmp = $1
+                        unless tmp =~ /127\./
+                            ip = tmp
+                            break
+                        end
+                    end
+                }
+
+                ip
+            end
+        end
+        Facter.add(:hostname) do
             confine :kernel => :darwin, :kernelrelease => "R7"
             setcode do
                 %x{/usr/sbin/scutil --get LocalHostName}
             end
         end
-        Facter.add("IPHostnumber") do
+        Facter.add(:iphostnumber) do
             confine :kernel => :darwin, :kernelrelease => "R6"
             setcode do
                 %x{/usr/sbin/scutil --get LocalHostName}
             end
         end
-        Facter.add("IPHostnumber") do
+        Facter.add(:iphostnumber) do
             confine :kernel => :darwin, :kernelrelease => "R6"
             setcode do
                 ether = nil
@@ -970,16 +1008,16 @@ class Facter
             end
         end
 
-        Facter.add("ps") do
+        Facter.add(:ps) do
             setcode do 'ps -ef' end
         end
 
-        Facter.add("ps") do
+        Facter.add(:ps) do
             confine :operatingsystem => %w{FreeBSD NetBSD OpenBSD Darwin}
             setcode do 'ps -auxwww' end
         end
 
-        Facter.add("id") do
+        Facter.add(:id) do
             #confine :kernel => %w{Solaris Linux}
             confine :operatingsystem => %w{Solaris Linux Fedora RedHat CentOS SuSE Debian Gentoo}
             setcode "whoami"
@@ -987,24 +1025,38 @@ class Facter
 
         locals = []
 
-        # Now see if we can find any other facts
+        # Now find all our loadable facts
+        factdirs = [] # All the places to check for facts
+        
+        # See if we can find any other facts in the regular Ruby lib
+        # paths
         $:.each do |dir|
             fdir = File.join(dir, "facter")
             if FileTest.exists?(fdir) and FileTest.directory?(fdir)
-                Dir.glob("#{fdir}/*.rb").each do |file|
-                    # Load here, rather than require, because otherwise
-                    # the facts won't get reloaded if someone calls
-                    # "loadfacts".  Really only important in testing, but,
-                    # well, it's important in testing.
-                    begin
-                        load file
-                    rescue => detail
-                        warn "Could not load %s: %s" %
-                            [file, detail]
-                    end
+                factdirs.push(fdir)
+            end
+        end
+        # Also check anything in 'FACTERLIB'
+        if ENV['FACTERLIB']
+            ENV['FACTERLIB'].split(":").each do |fdir|
+                factdirs.push(fdir)
+            end
+        end
+        factdirs.each do |fdir|
+            Dir.glob("#{fdir}/*.rb").each do |file|
+                # Load here, rather than require, because otherwise
+                # the facts won't get reloaded if someone calls
+                # "loadfacts".  Really only important in testing, but,
+                # well, it's important in testing.
+                begin
+                    load file
+                rescue => detail
+                    warn "Could not load %s: %s" %
+                        [file, detail]
                 end
             end
         end
+        
 
         # Now try to get facts from the environment
         ENV.each do |name, value|
@@ -1018,3 +1070,5 @@ class Facter
 
     Facter.loadfacts
 end
+
+# $Id: facter.rb 202 2007-03-21 16:44:44Z luke $
